﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace AirplaneWithRadar
{
    public partial class FormAirplaneConfig : Form
    {
        //переменная-выбранный самолёт
        DrawningAirplane _airplane = null;

        //событие
        private event Action<DrawningAirplane> EventAddAirplane;

        //конструктор
        public FormAirplaneConfig()
        {
            InitializeComponent();
            panelBlack.MouseDown += PanelColor_MouseDown;
            panelPurple.MouseDown += PanelColor_MouseDown;
            panelGray.MouseDown += PanelColor_MouseDown;
            panelGreen.MouseDown += PanelColor_MouseDown;
            panelRed.MouseDown += PanelColor_MouseDown;
            panelWhite.MouseDown += PanelColor_MouseDown;
            panelYellow.MouseDown += PanelColor_MouseDown;
            panelBlue.MouseDown += PanelColor_MouseDown;

            buttonCancel.Click += (object sender, EventArgs e) => Close();
        }

        //отрисовка самолёт
        private void DrawAirplane()
        {
            Bitmap bmp = new(pictureBox.Width, pictureBox.Height);
            Graphics gr = Graphics.FromImage(bmp);
            _airplane?.SetPosition(5, 5, pictureBox.Width, pictureBox.Height);
            _airplane?.DrawTransport(gr);
            pictureBox.Image = bmp;
        }

        //добавление события
        public void AddEvent(Action<DrawningAirplane> ev)
        {
            if (EventAddAirplane == null)
            {
                EventAddAirplane = new Action<DrawningAirplane>(ev);
            }
            else
            {
                EventAddAirplane += ev;
            }
        }

        private void LabelObject_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Label).DoDragDrop((sender as Label).Name, DragDropEffects.Move | DragDropEffects.Copy);
        }

        //проверка получаемой информации (ее типа на соответствие требуемому)
        private void PanelObject_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.Text))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        //действия при приеме перетаскиваемой информации
        private void PanelObject_DragDrop(object sender, DragEventArgs e)
        {
            switch (e.Data.GetData(DataFormats.Text).ToString())
            {
                case "labelSimpleObject":
                    _airplane = new DrawningAirplane((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, labelBaseColor.BackColor);
                    break;
                case "labelModifiedObject":
                    _airplane = new DrawningAirplaneWithRadar((int)numericUpDownSpeed.Value, (int)numericUpDownWeight.Value, labelBaseColor.BackColor, labelDopColor.BackColor,
                        checkBoxAddRadar.Checked, checkBoxAddLadder.Checked, checkBoxAddWindow.Checked);
                    break;

                case null:
                    throw new AirplaneNotFoundException(0);
                    break;
            }

            DrawAirplane();
        }

        //отправляем цвет с панели
        private void PanelColor_MouseDown(object sender, MouseEventArgs e)
        {
            (sender as Control).DoDragDrop((sender as Control).BackColor, DragDropEffects.Move | DragDropEffects.Copy);
        }

        //проверка получаемой информации (её типа на соответсвие требуемому)
        private void LabelColor_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }

        //принимаем основной цвет
        private void LabelBaseColor_DragDrop(object sender, DragEventArgs e)
        {
            //проверка на пустоту объекта
            if (_airplane != null)
            {
                _airplane.Airplane.BodyColor = (Color)e.Data.GetData(typeof(Color));

                DrawAirplane();
            }
        }

        //принимаем дополнительный цвет
        private void LabelAddColor_DragDrop(object sender, DragEventArgs e)
        {
            //проверка на пустоту объекта и правильную сущноть
            if (_airplane != null && _airplane.Airplane is EntityAirplaneWithRadar entityAirplane)
            {
                entityAirplane.DopColor = (Color)e.Data.GetData(typeof(Color));

                DrawAirplane();
            }
        }

        //добавление машины
        private void ButtonAddObject_Click(object sender, EventArgs e)
        {
            EventAddAirplane?.Invoke(_airplane);
            Close();
        }
    }
}
